@@ -20,7 +20,7 @@ function(w, klass = "GtkWidget", nullOk = FALSE, critical = TRUE)
    if(is.character(critical))
      stop(critical)
    else if(is.logical(critical) && critical)
-     stop(paste("object of class", class(w), "isn't a",klass))
+     stop(paste("object of class", paste(class(w), collapse = ", "), "isn't a", klass))
  }
 
  return(TRUE)
@@ -46,15 +46,18 @@ function(name, ..., PACKAGE = "RGtk2")
    #print(paste("Calling", name, "with args:", paste(..., collapse=", ")))
     val <- .Call(name, ..., PACKAGE = PACKAGE)
 	if (getOption("gdkFlush")) {
-		.Call("S_gdk_flush", PACKAGE = PACKAGE)
+		.Call("S_gdk_flush", PACKAGE = "RGtk2")
 	}
     val
 }
 
+
+######## BIT FLAG HANDLING ##########
+
 # Coerce something to a "flag" that can be operated on bitwise
 as.flag <- function(x) {
-	if (!is.numeric(x) && mode(x) != "raw")
-		stop("Flags must be numeric or raw")
+	if (!is.numeric(x))
+		stop("Flags must be numeric")
 	class(x) <- "flag"
 	x
 }
@@ -92,45 +95,24 @@ function(x)
 	class(x) <- "raw"
 	x
 }
-
-"==.flag" <-
-function(x, y) {
-	if (!.flagCompare(x, y))
-		.flagCompare(y, x)
-	else TRUE
-}
-
-.flagCompare <- function(x, y) {
-	if (is.character(y) && class(x)[1] != "flag") {
-		flags <- get(class(x)[1])
-		names(flags)[match(x, flags)] == y
-	} else if (is.logical(y)) {
-		x <- as.logical(x)
-		if (y) any(x)
-		else !any(x)
-	} else {
-		x <- .toBits(x)
-		y <- .toBits(y)
-		all(as.logical(sapply(1:length(x), function(i) x[i] == y[i])))
-	}
+.fromBits <- function(x)
+{
+  sum(as.integer(x) * c(2 ^ (0:30), -2^31)) 
 }
 
 # these actually perform the bit ops, after coercing args to bits
 .bitAnd <- function(x, y)
 {
-	x <- .toBits(x)
-	y <- .toBits(y)
-  x & y
+  .fromBits(.toBits(x) & .toBits(y))
 }
 .bitOr <- function(x, y)
 {
-  x <- .toBits(x)
-	y <- .toBits(y)
-  x | y
+  .fromBits(.toBits(x) | .toBits(y))
 }
 .bitNot <- function(x) {
-	x <- .toBits(x)
-	rawToBits(!x)[seq(1, 256, by=8)]
+  -1 - x
+  #x <- .toBits(x)
+	#.fromBits(rawToBits(!x)[seq(1, 256, by=8)])
 }
 
 "==.enum" <-
@@ -143,7 +125,8 @@ function(x, y)
   else if (inherits(y, "enum"))
     ans <- names(get(class(y)[1]))[y+1] == x
   
-  class(x) <- class(y) <- ""
+  x <- unclass(x)
+  y <- unclass(y)
   
   if (!ans)
     ans <- x == y
@@ -173,4 +156,64 @@ imagefile <- function(name)
 	.Call("convertPtrToNumeric", ptr, PACKAGE="RGtk2")
 }
 
+# Text manipulation
+
+.collapseClassName <-
+  #
+  # converts a class name of the form GtkButton
+  # to gtk_button.
+  # Also handles GtkCList to gtk_clist.
+  #
+function(name)
+{
+  tmp <- gsub("([ABCDEFGHIJKLMNOPQRSTUVWXYZ]+)", "_\\1", name)
+  tmp <- tolower(substring(tmp, 2))
+  gsub("_([abcdefghijklmnopqrstuvwxyz])_","_\\1", tmp)
+}
+
+# make a new environment by copying an existing one or creating one from scratch
+.newEnv <- function(env, parent = emptyenv())
+{
+  if (is.environment(env))
+    .copyEnv(env, parent)
+  else .as.environment(env, parent)
+}
+
+# make a copy of an environment
+# the new root environment is given 'parent' as its parent
+.copyEnv <- function(env, parent = emptyenv(), pattern)
+{
+  #p <- parentenv(env)
+  #if (recursive && p != emptyenv())
+  #  parent <- .copyEnv(p, parent, TRUE)
+  syms <- ls(env, pattern)
+  elements <- structure(lapply(syms, get, env), names = syms)
+  .as.environment(elements, parent)
+}
+
+# Coerce a list (or something coerceable to a list) to an environment
+.as.environment <- function(l, parent = emptyenv())
+{
+  if (is.environment(l))
+    return(l)
+  l <- as.list(l)
+  e <- new.env(TRUE, parent)
+  sapply(names(l), function(name) assign(name, l[[name]], e))
+  e
+}
+
+# assigning and removing elements of a structure into/from an environment
+# most efficient when 'elements' is an environment
+.massign <- function(elements, env)
+{
+  e <- .as.environment(elements)
+  sapply(ls(e), function(name) 
+    assign(name, get(name, e), env))
+}
+.mrm <- function(elements, env)
+{
+  e <- .as.environment(elements)
+  eapply(e, rm, env)
+}
+
 # .Call("R_getSignalNames", gtkButton())
